# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import Splitted, String


HEADER = (
    "Sample ID:",
    "Sample Type:",
    "Acquisition Date/Time-Intl.:"
)


class PerkinElmerBase(Splitted):
    SEPARATOR = "\n"


class PerkinElmerResult(object):
    FIELDS = [
        String("code_test"),
        String("concentration"),
        String("unit")
    ]

    def __init__(self, **kwargs):
        self.sample_id = kwargs.pop("sample_id")
        self.sample_type = kwargs.pop("sample_type")
        self.acquisition_date = kwargs.pop("acquisition_date")

    def parse(self, line):
        fields = self.FIELDS
        splitted = line.split(",")
        for index, field in enumerate(fields):
            setattr(self, field.name, splitted[index])


class PerkinElmerMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.RESULT: PerkinElmerResult
    }

    sample_id = None
    sample_type = None
    acquisition_date = None

    def add(self, type_message=None, **kwargs):
        kwargs["sample_id"] = self.sample_id
        kwargs["sample_type"] = self.sample_type
        kwargs["acquisition_date"] = self.acquisition_date
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not line:
                continue
            if "Analyte,Conc. Mean,Report Unit" in line:
                continue
            splitted = line.split(",")
            # Un résultat est composé de 3 champs (Analyte, Concentration,
            # Unit)
            if len(splitted) == 3:
                type_message = MessageV2.RESULT
                obj = self.add(type_message=type_message)
                obj.parse(line)
            # On ne récupère que les informations utiles dans l'entête dans
            # l'ordre d'apparition de ces informations
            elif len(splitted) == 2 and splitted[0] in HEADER:
                # Sample ID est en première position
                if HEADER.index(splitted[0]) == 0:
                    self.sample_id = splitted[1]
                # Sample Type se situe en seconde position
                elif HEADER.index(splitted[0]) == 1:
                    self.sample_type = splitted[1]
                # Acquisition Date en 3e position
                elif HEADER.index(splitted[0]) == 2:
                    self.acquisition_date = splitted[1]

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {
        "code": "perkinelmer",
        "label": "Spectro de Masse PerkinElmer Syngistix"
    }

    MESSAGE_TYPE_MAP = {
        MessageV2.RESULT: PerkinElmerResult,
    }

    def create(self):
        return PerkinElmerMultiple()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = PerkinElmerMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            if m.sample_type in ("Sample", "Standard"):
                sample = worklist.get_or_create(
                    sample_id=m.sample_id,
                    type="patient"
                )
            elif m.sample_type == "QC":
                sample = worklist.get_or_create(
                    control_lot=m.sample_id,
                    sample_id=m.sample_id,
                    type="control"
                )
            if m.concentration:
                sample.results.add(
                    code=m.code_test,
                    value=m.concentration,
                    unit=m.unit,
                    date=datetime.datetime.strptime(m.acquisition_date, "%d/%m/%Y %H:%M")
                )
        return worklist
